// centroid.cpp -- finds the spectral centroid of the given sample.

#include "centroid.h"
#include <fftw3.h>

bool centroid_initialized = false;
fftw_plan centroid_fft_plan;
double *centroid_fft_in;
std::complex<double> *centroid_fft_out;

void init_centroid()
{
	if (centroid_initialized)
		return;

	// FFTW_PATIENT is not worth it -- it takes about two hours to plan and gives very little
	// extra performance in practice.

	centroid_fft_in = reinterpret_cast<double *> (fftw_malloc(sizeof(double) * FEATURE_SAMPLES));
	centroid_fft_out = reinterpret_cast<std::complex<double> *> (fftw_malloc(sizeof(std::complex<double>) * (FEATURE_SAMPLES / 2 + 1)));
	centroid_fft_plan = fftw_plan_dft_r2c_1d(FEATURE_SAMPLES, centroid_fft_in, reinterpret_cast<fftw_complex *> (centroid_fft_out), FFTW_MEASURE | FFTW_DESTROY_INPUT);
	
	centroid_initialized = true;
}

void find_centroid(Sample &s)
{
	init_centroid();
	size_t N = std::min(s.samples.size(), (size_t)FEATURE_SAMPLES);

	// get the data, and pad
	for (unsigned i = 0; i < N; ++i) {
		centroid_fft_in[i] = s.samples[i];
	}
	for (unsigned i = N; i < FEATURE_SAMPLES; ++i) {
		centroid_fft_in[i] = 0.0;
	}

	// takes some time...
	fftw_execute(centroid_fft_plan);

	// find the centroid from the spectrum
	double weighted_sum = 0.0, total_sum = 0.0;
	for (unsigned i = 0; i < (FEATURE_SAMPLES / 2 + 1); ++i) {
		double f = (i + 0.5) * 22050.0 / (FEATURE_SAMPLES / 2 + 1);
		weighted_sum += f * abs(centroid_fft_out[i]);
		total_sum += abs(centroid_fft_out[i]);
	}

	std::vector<double> d;
	d.push_back(weighted_sum / total_sum);
	s.descriptors["centroid"] = d;
}

