// graph.cpp -- draws graphs that are semi-useful for debugging.

#include <stdio.h>
#include "graph.h"

void draw_graph(const char * const filename, std::vector<double> &coeffs)
{
	FILE *pgm = fopen(filename, "wb");
	fprintf(pgm, "P5\n%lu 200\n255\n", coeffs.size());
	
	double max = 0.0;
	for (unsigned j = 0; j < coeffs.size(); ++j) {
		max = std::max(max, fabs(coeffs[j]));
	}

	std::vector<unsigned> heights;
	for (unsigned x = 0; x < coeffs.size(); ++x) {
		int yy = 100 - (int)(100 * coeffs[x] / max);
		yy = std::max(yy, 0);
		yy = std::min(yy, 200);

		heights.push_back(yy);
	}

	for (unsigned y = 0; y < 200; ++y) {
		for (unsigned x = 0; x < coeffs.size(); ++x) {
			if (x == 0) {
				putc((heights[x] == y) ? 255 : 0, pgm);
			} else if (heights[x] > heights[x-1]) {
				if (y <= heights[x] && y >= heights[x-1])
					putc(255, pgm);
				else
					putc(0, pgm);
			} else {
				if (y >= heights[x] && y <= heights[x-1])
					putc(255, pgm);
				else
					putc(0, pgm);
			}
		}
	}

	fclose(pgm);
}
