// vorbis.cpp -- reads in Vorbis files, using libvorbisfile.

#include <stdio.h>
#include <vorbis/vorbisfile.h>
#include <cassert>

#include "vorbis.h"

void read_vorbis_file(Sample &s, char *filename)
{
	int junk __attribute__ ((unused));
	OggVorbis_File vf;
	unsigned num_read = 0;

	s.samples.reserve(5000000);

	FILE *fle = fopen(filename, "rb");
	if (fle == NULL) {	
		perror(filename);
		exit(1);
	}

	if (ov_open(fle, &vf, NULL, 0) < 0) {
		fprintf(stderr, "Couldn't open bitstream\n");
		exit(1);
	}

	for ( ;; ) {
		char buf[4096];
		short *ptr = (short *)buf;

		int ret = ov_read(&vf, buf, 4096, 0, 2, 1, &junk);
		if (ret < 0) {
			fprintf(stderr, "Error in Vorbis bitstream\n");
			exit(1);
		}
		if (ret == 0)
			break;

		assert(ret % 4 == 0);
		num_read += ret / 4;

		for (int i = 0; i < ret / 2; i += 2) {
			s.samples.push_back(0.5 * (double(ptr[i]) + double(ptr[i+1])));
		}

		if (num_read >= FEATURE_SAMPLES)
			break;
	}

	s.length = ov_time_total(&vf, -1);
	ov_clear(&vf);
}

