#include <stdio.h>
#include <assert.h>

#include "wave.h"

#define READ_BLOCKS 1024
#define WAVE_HEADER_SIZE 44

void read_wave_file(Sample &s, char *filename)
{
    unsigned file_pos = 0;
    long file_len;
    long data_size;
    signed short *data;
    FILE *fp;

    fp = fopen(filename, "rb");
    assert(fp != NULL);
    fseek(fp, 0, SEEK_END);
    file_len = ftell(fp);
    assert(file_len > FEATURE_SAMPLES * 2 + WAVE_HEADER_SIZE);


    long samples = (file_len - WAVE_HEADER_SIZE) / (2 * 2);
    s.samples.reserve(FEATURE_SAMPLES);

    fseek(fp, 44, SEEK_SET);
    data = (signed short *)malloc(sizeof(signed short) * READ_BLOCKS);
    while ((!feof(fp)) && file_pos < FEATURE_SAMPLES * 2)
    {
        data_size = fread(data, sizeof(signed short), READ_BLOCKS, fp);
        // data_size - 1 because we need to be able to read at least 2 samples
        for (int i = 0; i < data_size - 1; i += 2)
        {
            signed short l, r;
            l = data[i    ];
            r = data[i + 1];

#if 0
                /*
                 * There's a lot of codec delay in MP3. This skips
                 * the first 1105 samples (LAME's estimate of the
                 * delay for our test MP3s) to make lining up plots
                 * easier.
                 */
                static unsigned delayed = 0;
                if (++delayed < 1105)
                    continue;
#endif

            s.samples.push_back(0.5 * (double(r) + double(l)));
        }
        file_pos += data_size;
    }
    s.length = (samples / 44100.0);
    free(data);
}
