#include "vector_pairs.h"

int run_tests(char *dblk, int *hashes, char **names, 
                int vecs, int *thres, char delim) {
    /* compare all pairs of vectors for possible matches */
    int i, j, k, bitcount[256], type1_err=0, type2_err=0;
#ifdef INSERT_INTO_DB
    int ixFile = 0;
    int ixFile2 = 0;
    char *temp;
#endif
    char terminator;
    struct stat stat_results;
    if (delim == '\0') 
    {
        terminator = '\0';
    }
    else 
    {
        terminator = '\n';
    }
    setup_bitcount_tbl(bitcount);
    for (i = 0; i < vecs; i++)  
    {
        /* code for mysql database insertion */

#ifdef INSERT_INTO_DB
        ixFile = db_insert_file(names[i], i, vecs);
        temp = (char *) malloc(VECTOR_BYTES);
        strncpy(temp, dblk + i * MULTI_VEC_LEN + 0 * VECTOR_BYTES, VECTOR_BYTES);
        db_insert_fingerprint(ixFile, 1, temp, VECTOR_BYTES); /* fdmf_0 */
        strncpy(temp, dblk + i * MULTI_VEC_LEN + 1 * VECTOR_BYTES, VECTOR_BYTES);
        db_insert_fingerprint(ixFile, 2, temp, VECTOR_BYTES); /* fdmf_1 */
        strncpy(temp, dblk + i * MULTI_VEC_LEN + 2 * VECTOR_BYTES, VECTOR_BYTES);
        db_insert_fingerprint(ixFile, 3, temp, VECTOR_BYTES); /* fdmf_2 */
        free(temp);
#endif

        for (j = i + 1; j < vecs; j++) {
#ifdef INSERT_INTO_DB
            ixFile2 = db_insert_file(names[j], j, vecs);
#endif
            int distance[NUM_TESTS], score, test;
            char *ptr_a = dblk + i * MULTI_VEC_LEN;
            char *ptr_b = dblk + j * MULTI_VEC_LEN;
            int basenames_match, contents_similar;    
            /* calculate distance between two vectors for each test*/
            for (score = test = 0; test < NUM_TESTS; test++) {
                for (distance[test] = k = 0; distance[test] <= thres[test] && k < VECTOR_BYTES; k++) {
                    unsigned char c = *(ptr_a + k) ^ *(ptr_b + k);
                    distance[test] += bitcount[c];
                }
                ptr_a += VECTOR_BYTES;
                ptr_b += VECTOR_BYTES;

                if (distance[test] < thres[test])
                {
                    score++;
#ifdef INSERT_INTO_DB
                    db_add_score(ixFile, ixFile2, test+1, distance[test], thres[test]);
#endif
                }
            } /* post: score contains the number of tests that passed */
            contents_similar = (score >= SCORE_THRESHOLD) ? 1 : 0;
            basenames_match = (hashes[i] == hashes[j]) ? 2 : 0;
#ifdef IGNORE_GHOST_FILES
            /* If either file of a pair does not exist, ignore the pair */
            if (contents_similar + basenames_match) {
                if (stat(names[i], &stat_results)) continue;
                if (stat(names[j], &stat_results)) continue;
            }
# endif
            switch (contents_similar + basenames_match) {
                case 0: /* different contents, different basenames */
                    /* do nothing */
                    break;
                case 1: /* similar contents, different basenames */
                      printf("%s%c%s%c", names[i], delim, names[j], terminator);
                    type1_err++;    
                    break;
                case 2: /* different contents, same basenames */
                  /*    printf("type 2: %s%c%s%c", names[i], delim, 
                                names[j], terminator); */
                    type2_err++;
                    break;
                case 3: /* similar contents, same basenames */
                      printf("%s%c%s%c", names[i], delim, names[j], terminator);
                    break;
            }
            if (contents_similar + basenames_match == 1 ||
                contents_similar + basenames_match == 3)
            {
                printf("%d%c%d%c%d%c", distance[0], delim, distance[1], delim, distance[2], delim);
            }

        }
    }
    fprintf(stderr, "%d\t%d\t%d\t", thres[0], thres[1], thres[2]);
    fprintf(stderr, "%d\t%d\n", type1_err, type2_err); 
    return(0);
}
