#include "db.h"

#define SQL_INSERT_SCORE "INSERT INTO tblScore (ixFile1, ixFile2, ixType, dblScore) VALUES (?, ?, ?, 1 - (? / ?))"
#define SQL_SELECT_FINGERPRINT "SELECT ixFile FROM tblFingerprint WHERE ixFile = ? AND ixType = ?"
#define SQL_INSERT_FINGERPRINT "INSERT INTO tblFingerprint (ixFile, ixType, sFingerprint) VALUES (?, ?, ?)"
#define SQL_SELECT_FILE "SELECT ixFile FROM tblFile WHERE md5Path = MD5(?)"
#define SQL_INSERT_FILE "INSERT INTO tblFile(sPath, md5Path, cbFile) VALUES (?, MD5(?), ?)"

MYSQL *db_conn;
MYSQL_STMT *sqlInsertFile;
MYSQL_STMT *sqlSelectFingerprint;
MYSQL_STMT *sqlInsertFingerprint;
MYSQL_STMT *sqlSelectFile;
MYSQL_STMT *sqlInsertScore;

/*
 * Lets create a local cache of every ixFile value we get from
 * MySQL, so we won't have to query for it every time. 
 */
int *file_cache;
int file_cache_ready = 0;

int db_setup()
{
	char *server 	= "localhost";
	char *user   	= "master";
	char *password	= "7aBPK8huJQz3TNQS";
	char *database	= "master";

	db_conn = mysql_init(NULL);
	if (!mysql_real_connect(db_conn, server, user, password, database, 0, NULL, 0))
	{
		fprintf(stderr, "%s\n", mysql_error(db_conn));
		exit(0);
	}

	sqlInsertScore = mysql_stmt_init(db_conn);	
	mysql_stmt_prepare(sqlInsertScore, SQL_INSERT_SCORE, strlen(SQL_INSERT_SCORE));

	sqlSelectFingerprint = mysql_stmt_init(db_conn);
	mysql_stmt_prepare(sqlSelectFingerprint, SQL_SELECT_FINGERPRINT, strlen(SQL_SELECT_FINGERPRINT));
	
	sqlInsertFingerprint = mysql_stmt_init(db_conn);
	mysql_stmt_prepare(sqlInsertFingerprint, SQL_INSERT_FINGERPRINT, strlen(SQL_INSERT_FINGERPRINT));
	
	sqlSelectFile = mysql_stmt_init(db_conn);
	mysql_stmt_prepare(sqlSelectFile, SQL_SELECT_FILE, strlen(SQL_SELECT_FILE));

	sqlInsertFile = mysql_stmt_init(db_conn);
	mysql_stmt_prepare(sqlInsertFile, SQL_INSERT_FILE, strlen(SQL_INSERT_FILE));

	return(0);
}

void db_close()
{
	mysql_close(db_conn);

	mysql_stmt_close(sqlInsertScore);
	mysql_stmt_close(sqlSelectFingerprint);
	mysql_stmt_close(sqlInsertFingerprint);
	mysql_stmt_close(sqlSelectFile);
	mysql_stmt_close(sqlInsertFile);
}

void init_file_cache(int size)
{
	int i;
	if (!file_cache_ready)
	{
		file_cache = (int*) malloc(sizeof(int) * size);
		for (i = 0; i < size; i++)
			file_cache[i] = -1;
/*		memset(file_cache, 0, sizeof(int) * size); */
		file_cache_ready = 1;
	}
}

int db_find_file(char *file, int cFileNo)
{
	int ixFile = -1;
    MYSQL_BIND bind[1];
	unsigned long length;
	my_bool is_null;
	my_bool error;
	
	if (file_cache[cFileNo] != -1)
		return file_cache[cFileNo];
		
	memset(bind, 0, sizeof(bind));
	
	length = strlen(file);
	
	bind[0].buffer_type = MYSQL_TYPE_STRING;
	bind[0].buffer = file;
	bind[0].buffer_length = strlen(file);
	bind[0].is_null = 0;
	bind[0].length = &length;
	
	if (mysql_stmt_bind_param(sqlSelectFile, bind))
	{
		fprintf(stderr, "mysql_stmt_bind_param() failed in db_find_file()\n");
		fprintf(stderr, " %s\n", mysql_stmt_error(sqlSelectFile));
		exit(0);
	}
	if (mysql_stmt_execute(sqlSelectFile))
	{
		fprintf(stderr, "mysql_stmt_execute() failed in db_find_file()\n");
		fprintf(stderr, " %s\n", mysql_stmt_error(sqlSelectFile));
		exit(0);
	}

	memset(bind, 0, sizeof(bind));

	bind[0].buffer_type= MYSQL_TYPE_LONG;
	bind[0].buffer = (char *)&ixFile;
	bind[0].buffer_length = 4;
	bind[0].length = &length;
	bind[0].is_null = &is_null;
	bind[0].error = &error;
	
	if (mysql_stmt_bind_result(sqlSelectFile, bind))
	{
		fprintf(stderr, "mysql_stmt_bind_result() failed in db_find_file()\n");
		fprintf(stderr, " %s\n", mysql_stmt_error(sqlSelectFile));
		exit(0);
	}

	if (mysql_stmt_store_result(sqlSelectFile))
	{
		fprintf(stderr, "mysql_stmt_store_result() failed in db_find_file()\n");
		fprintf(stderr, " %s\n", mysql_stmt_error(sqlSelectFile));
		exit(0);
	}

	if (mysql_stmt_num_rows(sqlSelectFile) == 0)
		return -1;

	if (mysql_stmt_fetch(sqlSelectFile))
	{
		fprintf(stderr, "mysql_stmt_fetch() failed in db_find_file()\n");
		fprintf(stderr, " %s\n", mysql_stmt_error(sqlSelectFile));
		exit(0);
	}
	if (ixFile != -1)
		file_cache[cFileNo] = ixFile;
	
	/* fprintf(stderr, "%s %d %ld %d\n", file, ixFile, length, error); */
	return ixFile;
}

/* returns ixFile, -1 if error */
int db_insert_file(char *file, int cFileNo, int cFiles)
{
	struct stat fileBuf;
	int ixFile = -1;
	long cbFile = 0;
	MYSQL_BIND bind[3];
	unsigned long str_len;
	
	init_file_cache(cFiles);

	ixFile = db_find_file(file, cFileNo);
	if (ixFile != -1)
		return ixFile;

	memset(bind, 0, sizeof(bind));

	stat(file, &fileBuf);
	cbFile = fileBuf.st_size;
	
	str_len = strlen(file);

	bind[0].buffer_type = MYSQL_TYPE_STRING;
	bind[0].buffer = (char *)file;
	bind[0].buffer_length = strlen(file);
	bind[0].length = &str_len;
	bind[0].is_null = 0;
	
	// yes, we're duping info, let mysql handle MD5 hashing
	bind[1].buffer_type = MYSQL_TYPE_STRING;
	bind[1].buffer = (char *)file;
	bind[1].buffer_length = strlen(file);
	bind[1].length = &str_len;
	bind[1].is_null = 0;
	
	bind[2].buffer = (char *)&cbFile;
	bind[2].buffer_type = MYSQL_TYPE_LONG;
	bind[2].is_null = 0;
	bind[2].length = 0;

	if (mysql_stmt_bind_param(sqlInsertFile, bind))
	{
		fprintf(stderr, "mysql_stmt_bind_param() failed in db_insert_file()\n");
  		fprintf(stderr, " %s\n", mysql_stmt_error(sqlInsertFile));
  		exit(0);
	}

	if (mysql_stmt_execute(sqlInsertFile))
	{
		fprintf(stderr, "mysql_stmt_execute failed in db_insert_file()!\n");
  		fprintf(stderr, " %s\n", mysql_stmt_error(sqlInsertFile));		
		exit(0);
	}
	
	/* return db_find_file(file); */
	return mysql_insert_id(db_conn);
}

int db_has_fingerprint(int ixFile, int ixType)
{
	int ixReturn = -1;
    MYSQL_BIND bind[2];
	unsigned long length;
	my_bool is_null;
	my_bool error;
	
	memset(bind, 0, sizeof(bind));
	
	bind[0].buffer_type = MYSQL_TYPE_LONG;
	bind[0].buffer = (char *)&ixFile;
	bind[0].is_null = 0;
	bind[0].length = 0;
	
	bind[0].buffer_type = MYSQL_TYPE_LONG;
	bind[0].buffer = (char *)&ixType;
	bind[0].is_null = 0;
	bind[0].length = 0;
	
	if (mysql_stmt_bind_param(sqlSelectFingerprint, bind))
	{
		fprintf(stderr, "mysql_stmt_bind_param() failed in db_has_fingerprint()\n");
		fprintf(stderr, " %s\n", mysql_stmt_error(sqlSelectFingerprint));
		exit(0);
	}
	if (mysql_stmt_execute(sqlSelectFingerprint))
	{
		fprintf(stderr, "mysql_stmt_execute() failed in db_has_fingerprint()\n");
		fprintf(stderr, " %s\n", mysql_stmt_error(sqlSelectFingerprint));
		exit(0);
	}

	memset(bind, 0, sizeof(bind));

	bind[0].buffer_type= MYSQL_TYPE_LONG;
	bind[0].buffer = (char *)&ixReturn;
	bind[0].buffer_length = 4;
	bind[0].length = &length;
	bind[0].is_null = &is_null;
	bind[0].error = &error;
	
	if (mysql_stmt_bind_result(sqlSelectFingerprint, bind))
	{
		fprintf(stderr, "mysql_stmt_bind_result() failed in db_has_fingerprint()\n");
		fprintf(stderr, " %s\n", mysql_stmt_error(sqlSelectFingerprint));
		exit(0);
	}

	if (mysql_stmt_store_result(sqlSelectFingerprint))
	{
		fprintf(stderr, "mysql_stmt_store_result() failed in db_has_fingerprint()\n");
		fprintf(stderr, " %s\n", mysql_stmt_error(sqlSelectFingerprint));
		exit(0);
	}

	if (mysql_stmt_num_rows(sqlSelectFingerprint) == 0)
		return 0;
	else 
		return 1;
	
	/*
	char *sqlSelect;
	MYSQL_RES *res;
	MYSQL_ROW row;
	int ret;
	sqlSelect = (char *) malloc(2048);
	sprintf(sqlSelect, "SELECT ixFile FROM tblFingerprint WHERE ixFile = %d AND ixType = %d", ixFile, ixType);
	mysql_query(db_conn, sqlSelect);
	res = mysql_use_result(db_conn);
	if ((row = mysql_fetch_row(res)) != NULL)
		ret = 1;
	else
		ret = 0;

	free(sqlSelect);
	mysql_free_result(res);
	return ret;
	*/
}

void db_insert_fingerprint(int ixFile, int ixType, char *fingerprint, int fingerprint_length)
{
	MYSQL_BIND bind[3];
	unsigned long str_len;

	if (db_has_fingerprint(ixFile, ixType))
		return;
		
	memset(bind, 0, sizeof(bind));

	str_len = fingerprint_length;
	
	bind[0].buffer_type = MYSQL_TYPE_LONG;
	bind[0].buffer = (char *)&ixFile;
	bind[0].length = 0;
	bind[0].is_null = 0;
	
	// yes, we're duping info, let mysql handle MD5 hashing
	bind[1].buffer_type = MYSQL_TYPE_LONG;
	bind[1].buffer = (char *)&ixType;
	bind[1].length = 0;
	bind[1].is_null = 0;
	
	bind[2].buffer_type = MYSQL_TYPE_BLOB;
	bind[2].buffer = fingerprint;
	bind[2].buffer_length = fingerprint_length;
	bind[2].length = &str_len;
	bind[2].is_null = 0;
	bind[2].is_unsigned = 1;

	if (mysql_stmt_bind_param(sqlInsertFingerprint, bind))
	{
		fprintf(stderr, "mysql_stmt_bind_param() failed in db_insert_fingerprint()\n");
  		fprintf(stderr, " %s\n", mysql_stmt_error(sqlInsertFingerprint));
  		exit(0);
	}

	if (mysql_stmt_execute(sqlInsertFingerprint))
	{
		fprintf(stderr, "mysql_stmt_execute failed in db_insert_fingerprint()!\n");
  		fprintf(stderr, " %s\n", mysql_stmt_error(sqlInsertFingerprint));		
		exit(0);
	}
	
/*	char *sqlInsert;
	char *sFingerprint;
	unsigned long cbFingerprint, i;

	if (db_has_fingerprint(ixFile, ixType))
		return;

	sqlInsert = (char *) malloc(2048);
	sFingerprint = (char *) malloc(2048);
	cbFingerprint = mysql_real_escape_string(db_conn, sFingerprint, fingerprint, fingerprint_length);
	
	
	sprintf(sqlInsert, "INSERT INTO tblFingerprint (ixFile, ixType, sFingerprint) VALUES (%d, %d, '%s')", ixFile, ixType, sFingerprint);
	mysql_real_query(db_conn, sqlInsert);
*/
}

void db_add_score(int ixFile1, int ixFile2, int ixType, int bDistance, int bThreshold)
{
	MYSQL_BIND bind[5];
	int i;

	memset(bind, 0, sizeof(bind));

	for (i = 0; i < 5; i++)
	{
		bind[i].buffer_type = MYSQL_TYPE_LONG;
		bind[i].is_null = 0;
		bind[i].length = 0;
	}

	bind[0].buffer = (char *)&ixFile1;
	bind[1].buffer = (char *)&ixFile2;
	bind[2].buffer = (char *)&ixType;
	bind[3].buffer = (char *)&bDistance;
	bind[4].buffer = (char *)&bThreshold;

	if (mysql_stmt_bind_param(sqlInsertScore, bind))
	{
		fprintf(stderr, "mysql_stmt_bind_param failed in db_add_score()!\n");
		fprintf(stderr, " %s\n", mysql_stmt_error(sqlInsertScore));
		exit(0);		
	}

	if (mysql_stmt_execute(sqlInsertScore))
	{
	/* failing this insert is perfectly legitimate, as this comparison may already
	 * exist somewhere in the database 
	 * */
	/*
		fprintf(stderr, "mysql_stmt_execute failed in db_add_score()!\n");
		fprintf(stderr, " %s\n", mysql_stmt_error(sqlInsertScore));
		exit();
	*/
	}
}
