#include "foo.h"

#define SAMPLE_RATE 44100
#define CHANNELS 2
#define N_BLOCKS 1024

t_fooid *fooid;

int fsize(char *name)
{
    struct stat stbuf;
    
    if (stat(name, &stbuf) == -1)
    {
        fprintf(stderr, "fsize: can't access %s\n", name);
        return 0;    
    }    
    return stbuf.st_size;
}

int main(int argc, char *argv[])
{
    FILE *fp;
    signed short *data;
    int fp_more = 1;
    int fp_data_size;
    unsigned long fp_fingerprint_size;
    unsigned char *fp_fingerprint;
    int file_size;
    int audio_length;
    int whereami, ixFile;
    
    if (argc == 1 || argc > 3)
    {
        fprintf(stderr, "Usage:\n");
        fprintf(stderr, "\t%s <raw-audiofile> [<original-filename>]:\n", argv[0]);
        exit(1);
    }
    else
    {
        if ((fp = fopen(argv[1], "rb")) == NULL)
        {
            fprintf(stderr, "Can't open %s\n", argv[1]);
            exit(1);
        }
        else
        {
            data = (signed short *)malloc(sizeof(signed short) * N_BLOCKS);
            fooid = fp_init(SAMPLE_RATE, CHANNELS);
            if (ferror(fp))
            {
                fprintf(stderr, "Error %d!\n", ferror(fp));    
            }

            /*
             * Lets skip the header (we know we get 44100 Hz, 16 bit, 2 channel)
             */
            fseek(fp, 44, SEEK_SET);
            whereami = 44;

            while ((!feof(fp)) && (fp_more == 1) && !ferror(fp))
            {
                fp_data_size = fread(data, sizeof(signed short), N_BLOCKS, fp);
                whereami += fp_data_size;
                fp_more = fp_feed_short(fooid, data, fp_data_size);
            }
            fclose(fp);
            
            fp_fingerprint_size = fp_getsize(fooid);
            
            file_size = fsize(argv[1]);
            
            /*
             * The * 2 is because it is 16 bits, while the file size is in 8 bits 
             */
            audio_length = (file_size / (float) (SAMPLE_RATE * CHANNELS * 2)) * 100; 
            
            fp_fingerprint = (unsigned char *)malloc(sizeof(unsigned char) * fp_fingerprint_size);
            if (fp_calculate(fooid, audio_length, fp_fingerprint) < 0)
            {
                fprintf(stderr, "fp_calculate failed!\n");
            }

            if (argc == 3)
            {
                db_setup();
                
                ixFile = db_insert_file(argv[2], 0, 1);
                
                db_insert_fingerprint(ixFile, 4, (char *) fp_fingerprint, fp_fingerprint_size);
                
                db_close();
            }

            fp_free(fooid);
            free(data);    
            free(fp_fingerprint);
        }
    }
    
    return 0;
}
