<?php
/**
 * This script display potential duplicates in the database,
 * and highlights the duplicates that match by file name,
 * or have been verified to be correct.
 *
 * Options:
 *   fVerified=1        -- show only verified duplicates
 *   fVerified=0        -- show only non-verified duplicates
 *   fMatch=1           -- show only matching file names
 *   fMatch=0           -- show only non-matching file names
 *   ixPage=<page>      -- skip <page> * 1000 results
 *
 * If fVerified and fMatch is not specified, no restrictions
 * are placed on the duplicates status.
 *
 * @author Vegard Andreas Larsen <vegarl@stud.ntnu.no>
 */    
require_once('include/include.php');
$oDB = CDatabase::Get();

$cResultsPerPage = 1000;
$ixPage = 0;
$ixStart = 0;
$ixScoreMethod = 1;

require_once('duplicate_functions.php');

/**
 * Get a list of the verified duplicates.
 */
$sql = "SELECT * FROM tblVerifiedMatch";
$rgoVerified = $oDB->extended->GetAll($sql);
$rgVerified = array();
foreach ($rgoVerified as $o)
{
    $rgVerified[$o->ixFile1][$o->ixFile2] = true; 
}
$rgoVerified = null;
unset($rgoVerified);

$sqlSelectFile = "SELECT ixFile, sPath FROM tblFile";
$rgoFiles = $oDB->extended->GetAssoc($sqlSelectFile, null, null, null, 
    MDB2_FETCHMODE_ASSOC, false);
$rgoFiles = str_replace('/home/vegard/Music/', '', $rgoFiles);
$rgoFiles = str_replace('/mnt/media/', '', $rgoFiles);


/**
 * Now lets get all the scores from the database
 */

// Lower bound for values extracted.
$dblScoreLimit = 0.05;
if (isset($_GET['ixType']))
{
    $ixType = (int) $_GET['ixType'];
    $sqlSelectScores = "SELECT ixFile1, ixFile2, dblScore ".
        "FROM tblScore WHERE ".
        "dblScore > $dblScoreLimit AND ixType = $ixType ".
        "ORDER BY dblScore DESC LIMIT ?, ?";
}
elseif ($_GET['ixScoreMethod'])
{
    $ixScoreMethod = (int) $_GET['ixScoreMethod'];
    $sqlSelectScores = "SELECT ixFile1, ixFile2, dblScore ".
        "FROM tblSummedScore ".
        "WHERE dblScore > $dblScoreLimit ".
        "AND ixScoreMethod = $ixScoreMethod ".
        "ORDER BY dblScore DESC LIMIT ?, ?";
}
else
{
    /**
     * No type specified, display a menu of choices.
     */
    echo '<h1>Combined methods</h1>';
    $sql = "SELECT ixScoreMethod, sTitle FROM tblScoreMethods";
    $r = $oDB->extended->GetAll($sql);
    foreach ($r as $row)
    {
        echo '<a href="?ixScoreMethod='.$row->ixScoreMethod.'">'.
            $row->sTitle.'</a><br/>';
    }

    echo '<h1>Individual descriptors</h1>';
    $sql = "SELECT ixType, sType FROM tblType";
    $r = $oDB->extended->GetAll($sql);
    foreach ($r as $row)
    {
        echo '<a href="?ixType='.$row->ixType.'">'.
            $row->sType.'</a><br/>';
    }
    die();
}

/**
 * Allow parameters to change what is fetched.
 */
if ($_GET['ixPage'])
{
    $ixPage = (int) $_GET['ixPage'];
    if ($ixPage < 0) $ixPage = 0;
    $ixStart = $cResultsPerPage * $ixPage;
}

/**
 * Go fetch!
 */
$res = $oDB->extended->GetAll($sqlSelectScores, null, 
    array($ixStart, $cResultsPerPage), 
    array('integer', 'integer'));

/*
 * Output the HTML page headers
 */
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" 
  "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="no" lang="no">
  <head>
    <title>Master thesis</title>
    <link rel="stylesheet" type="text/css" href="css/main.css" />
  </head>
  <body>
<form method="post" action="save_verified.php">
<input type="hidden" name="redirect_to" 
  value="display_unverified_scores.php<?php 
echo (isset($ixType) ? "?ixType=$ixType" : "?ixScoreMethod=$ixScoreMethod"); 
?>" />
<input type="submit" value="Save verified" />
<table>
  <tr>
    <th><abbr title="Match number">M#</abbr></th>
    <th><abbr title="Do file names match?">M?</abbr></th>
    <th><abbr title="Is this match verified">V?</abbr></th>
    <th><abbr title="Check to verify">?</abbr></th>
    <th><abbr title="File numbers (ixFile)">F#</abbr></th>
    <th>Files</th>
    <th>Score</th>
  </tr>
<?

$j = $ixStart;
foreach ($res as $row)
{

    $fVerified = isset($rgVerified[$row->ixFile1][$row->ixFile2]);
    
    $fCompares = compare_filename(
        $rgoFiles[$row->ixFile1], 
        $rgoFiles[$row->ixFile2]);

    if ((!isset($_GET['fVerified']) ||
        ($_GET['fVerified'] == 1 && $fVerified) ||
        ($_GET['fVerified'] == 0 && !$fVerified)) 
        &&
        (!isset($_GET['fMatch']) ||
        ($_GET['fMatch'] == 1 && $fCompares) ||
        ($_GET['fMatch'] == 0 && !$fCompares)))
    {
        /*
         * Output ugly HTML table data.
         */
        echo '<tr class="'.($j % 2 == 0 ? 'colored' : '').'">';        
        echo '<td rowspan="2">'.($j+1).'</td>';
        echo '<td rowspan="2"><img src="images/'.
            ($fCompares ? "green" : "red").'" /></td>';
        echo '<td rowspan="2"><img src="images/'.
            ($fVerified ? "green" : "red").'.png" /></td>';
        echo '<td rowspan="2"><input type="checkbox" '.
            'name="match[]" value="'.$row->ixFile1.'-'.
            $row->ixFile2.'" /></td>';
        echo '<td>'.$row->ixFile1.'</td>';
        echo '<td align="right">'.$rgoFiles[$row->ixFile1].'</td>';
        echo '<td rowspan="2">'.($row->dblScore).'</td>';
        echo '</tr>';
        echo '<tr class="bottom '.($j % 2 == 0 ? 'colored' : '').'">';
        echo '<td>'.$row->ixFile2.'</td>';
        echo '<td align="right">'.$rgoFiles[$row->ixFile2].'</td>';    
        echo '</tr>';
        $j++;
    }
}

?>
</table>
<input type="submit" value="Save verified" />
</form>

</body>
</html>
