<?php
/**
 * This script contains a variety of functions for comparing two file
 * names against each other, and determining if they could be the same 
 * song.
 * 
 * @author Vegard Andreas Larsen <vegarl@stud.ntnu.no>
 */
function tokenize_filename($s)
{
    // remove anything before a space-hyphen-space sequence 
    // (often artist or album name)
    $s = preg_replace("/.* - /i", '', $s);
    $s = preg_replace("/[-\s\d]+/", ' ', strtolower($s));
    $rgTokens = split(' ', $s);
    foreach ($rgTokens as $key => $value)
    {
        if (strlen($value) <= 2)
            unset($rgTokens[$key]);
    }
    return $rgTokens;
}

function filename($s)
{
    return pathinfo($s, PATHINFO_FILENAME);
    //return $parts['filename'];
}

function artistname($s)
{
    $sDir = pathinfo($s, PATHINFO_DIRNAME);
    $rgDirs = explode('/', $sDir);
    return strtolower($rgDirs[3]);
}

function compare_filename($s1, $s2, $rgTokens1 = NULL, $rgTokens2 = NULL)
{
    if ($rgTokens1 == NULL)
        $rgTokens1 = tokenize_filename(filename($s1));
    if ($rgTokens2 == NULL)
        $rgTokens2 = tokenize_filename(filename($s2));
    $cHits = 0;
    $rgAllTokens = array_unique(array_merge($rgTokens1, $rgTokens2));
    foreach ($rgAllTokens as $token)
    {
        if (in_array($token, $rgTokens1) && in_array($token, $rgTokens2)) 
            $cHits++;
    }
    $cTokens = max(count($rgTokens1), count($rgTokens2));
    if ($cTokens < 1) return false;
    
    return ((artistname($s1) == artistname($s2) && 
        ($cHits / (float) $cTokens) > 0.6));        
}    
?>
