<?php
/**
 * This script finds the average -- and the minimum and maximum -- 
 * distance for verified duplicates with the simple descriptors
 * song length, centroid, msratio, steepness and zerocrossings.
 * It also lists the percentage of verified duplicates that will
 * be found when searching within 2 times the average distance
 * for a given descriptor.
 * 
 * @author Vegard Andreas Larsen <vegarl@stud.ntnu.no>
 */
require_once('include/include.php');
$oDB = CDatabase::Get();

/*
 * Get a list of descriptor names.
 */
$sql = "SELECT ixType, sType FROM tblType";
$rgsTypes = $oDB->extended->GetAssoc($sql, null, null, null, 
    MDB2_FETCHMODE_ASSOC, false);

/*
 * Find all the verified matches.
 */
$sql = "SELECT * FROM tblVerifiedMatch";
$rgoVerified = $oDB->extended->GetAll($sql);
$rgVerified = array();
foreach ($rgoVerified as $o)
{
    $rgVerified[$o->ixFile1][$o->ixFile2] = true;
    $rgVerified[$o->ixFile2][$o->ixFile1] = true;
}
$rgoVerified = null;
unset($rgoVerified);

function array_average($rg)
{
    return (array_sum($rg) / (float) count($rg));
}

/*
 * The descriptors to look at.
 */
$rgixType = array(6, 7, 8, 9, 10, 11);
foreach ($rgixType as $ixType)
{
    /*
     * Find all the fingerprints for the verified matches.
     */
    $sql = 
        "SELECT f1.sFingerprint AS fp1, f2.sFingerprint AS fp2, ixFile1 ".
        "FROM tblVerifiedMatch v, tblFingerprint f1, tblFingerprint f2 ".
        "WHERE f1.ixFile = v.ixFile1 AND f2.ixFile = v.ixFile2 AND ".
        "f1.ixType = ? AND f2.ixType = f1.ixType";
    $rgoFingerprint = $oDB->extended->GetAll($sql, null, 
        array($ixType), array('integer'));
    
    /*
     * Find the distance for each verified match.
     * This cannot be done in MySQL, because the column
     * that stores the fingerprints is in binary format,
     * to accomodate storing of all types of fingerprints,
     * and MySQL wouldn't know what to do with a
     * subtraction of binary data.
     */
    $rgDiff = array();
    foreach ($rgoFingerprint as $oF)
    {
        $rgDiff[] = abs($oF->fp1 - $oF->fp2);
    }
    sort($rgDiff);
    $avg = array_average($rgDiff);
    $min = $rgDiff[0];
    $max = round($rgDiff[count($rgDiff)-1], 2);

    /*
     * Now lets figure out how many fingerprints we would 
     * need to search (on average) when limiting our search 
     * to fingerprints within +- 2 * $avg, and how many of 
     * the results we find are verified.
     */    
    $sql = "SELECT ixFile, sFingerprint FROM tblFingerprint WHERE ixType = ?";
    $rgFingerprint = $oDB->extended->GetAll($sql, null, 
        array($ixType), array('integer'));
    $rgixSearchFor = array_keys($rgoFingerprint);
    $rgValues = array();
    $rgNumVerified = array();
    $delta = $avg * 2;
    foreach ($rgixSearchFor as $ix)
    {
        $dblValue = $rgoFingerprint[$ix]->fp1;
        $ixFile = $rgoFingerprint[$ix]->ixFile1;
        $c = 0;
        $cNumVerified = 0;
        foreach ($rgFingerprint as $oFingerprint)
        {
            $dblFingerprint = (float) $oFingerprint->sFingerprint;
            if ($dblFingerprint < ($dblValue + $delta) && 
                $dblFingerprint > ($dblValue - $delta))
            {
                $c++;
                $ixFile2 = (float) $oFingerprint->ixFile;
                if (isset($rgVerified[$ixFile][$ixFile2]))
                    $cNumVerified++;
            }
        }
        $rgValues[] = $c;
        $rgNumVerified[] = $cNumVerified / count($rgVerified[$ixFile]);
    }
    $c = round(array_average($rgValues), 0);
    $dblFindRate = round(array_average($rgNumVerified), 4);

    echo "avg: $avg\tmin: $min\tmax: $max\tsearch: $c\t"
        ."find rate: $dblFindRate\t\t".$rgsTypes[$ixType]."\n";

?>
