<?php
require_once('MDB2.php');
/**
 * A singleton database wrapper for MDB2.
 * 
 * Call CDatabase::Get() to get to the MDB2 database object.
 * This class handles everything related to setting up
 * and tearing down the database connection.
 * 
 * @author Vegard Andreas Larsen <vegard@prove.no>
 * @package prove
 */
class CDatabase
{
	private static $coDSN =  array(
		'phptype'  => DB_DBTYPE,
		'username' => DB_USER,
		'password' => DB_PASSWORD,
		'hostspec' => DB_HOST,
		'database' => DB_DATABASE,
		);

	private static $coOptions = array(
		'debug' => 2,
		'result_buffering' => false,
		'portability' => MDB2_PORTABILITY_NONE
		);


	private static $coDB;
	/**
	 * Returns a valid database object.
	 * 
	 * @return object MDB2 database connection object.
	 */
	public static function Get()
	{
		if (!isset(self::$coDB))
		{
			self::$coDB =& MDB2::factory(self::$coDSN, self::$coOptions);
			self::$coDB->setFetchMode(MDB2_FETCHMODE_OBJECT);
			self::$coDB->loadModule('Extended');
			self::$coDB->setCharset('UTF8');
			if (PEAR::isError(self::$coDB)) 
			{
				trigger_error(self::$coDB->getMessage());
			}
		}
		return self::$coDB;
	}

	/**
	 * Translate a PHP array to a MySQL array.
	 * 
	 * Takes an array, and translates it into a string that represents a SQL array.
	 * Key names are forgotten. You will have to add parenthesis yourself.
	 * Array(3 => 'one', 'two', 'three') becomes '\'one\', \'two\', \'three\''
	 * 
	 * @param array $rgPHP PHP array
	 * @return string MySQL array (without surrounding parenthesis)
	 */
	public static function phpArrayToSQLArray($rgPHP)
	{
			$db = self::Get();
			foreach ($rgPHP as $key => $item)
            {
				if ((int) $item == $item)
					$rgPHP[$key] = (int)$item;
				else
					$rgPHP[$key] = $db->quote($item);
			}
			return join(', ', $rgPHP);
	}

}

?>
