<?php
/**
 * This file compares the fingerprints produced by Gunderson's descriptors using
 * Euclidean distance, and generates a score for each fingerprint comparison.
 * 
 * Outputs  SQL insert statements to descriptor_inserts.sql that should
 * processed by MySQL at a later stage.
 * 
 * @author Vegard Andreas Larsen <vegarl@stud.ntnu.no>
 */
require_once('include/include.php');
ob_end_flush();

$rgMax = array(
    6 => 50.0,            // centroid
    7 => 5.0,            // time
    8 => 0.005,            // msratio
    9 => 25.0,            // steepness
    10 => 2000.0,        // zerocrossings
    11 => 1000.0,        // zerocrossing_guard
    12 => 40.0,            // mfcc_avg
    13 => 3.0,            // mfcc_delta_avg
    18 => 20.0,            // mfcc_f1_avg
    19 => 5.0,            // mfcc_f1_delta_avg

    14 => 50,            // mfcc_delta_moment2
    15 => 12500,        // mfcc_delta_moment3
    16 => 6000000,        // mfcc_delta_moment4
    17 => 4350000000,        // mfcc_delta_moment5
    
    20 => 1500,            // mfcc_f1_delta_moment2
    21 => 1250000,        // mfcc_f1_delta_moment3
    22 => 1175000000,        // mfcc_f1_delta_moment4
    23 => 1410000000000,    // mfcc_f1_delta_moment5
    
    24 => 6000,            // mfcc_delta_moment2
    25 => 6100000,        // mfcc_delta_moment3
    26 => 10000000000,        // mfcc_delta_moment4
    27 => 12800000000000,    // mfcc_delta_moment5
    
    28 => 2000,            // mfcc_moment2
    29 => 1700000,        // mfcc_moment3
    30 => 2900000000,        // mfcc_moment4
    31 => 4100000000000,    // mfcc_moment5
    );


$oDB = CDatabase::Get();

// we are ignoring a bunch of _delta_ moments
//$rgTypes = array_keys($rgMax);
$rgTypes = array(14, 15, 16, 17, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31);

//foreach ($rgTypes as $ixType)
//{
//    if (!isset($rgMax[$ixType]))
//        continue;
        
    $fp = fopen("inserts3.sql", "w");
    
    fwrite($fp, "INSERT DELAYED IGNORE INTO tblScore (ixFile1, ixFile2, ixType, dblScore) VALUES ");
    $fCommaFirst = false;
        
    $sqlSelectFingerprints = "SELECT ixFile, sFingerprint FROM tblFingerprint WHERE ixType = ? ORDER BY ixFile";
    $cbBefore = memory_get_usage(true);
    foreach ($rgTypes as $ixType)
    {
        // So we don't have to do sqrt() a few million times, we cache
        // the maximum Euclidean distance squared
        $dblMaxDistance[$ixType] = $rgMax[$ixType] * $rgMax[$ixType];
        $rgoFingerprints[$ixType] = $oDB->extended->GetAll($sqlSelectFingerprints, null, array($ixType), array('integer'), MDB2_FETCHMODE_ORDERED);
        foreach ($rgoFingerprints[$ixType] as $k => $f)
        {
            $rgoFingerprints[$ixType][$k][1] = explode(',', $rgoFingerprints[$ixType][$k][1]);
        }
    }
    $cbAfter = memory_get_usage(true);
    
    echo (($cbAfter - $cbBefore)/1024)."kB memory in use before starting\n";
    
    $cFingerprints = count($rgoFingerprints[$rgTypes[0]]);
    $cInserts = 0;
    for ($i = 0; $i < $cFingerprints; $i++)
    {
        echo "\ti=$i\n";
        for ($j = $i + 1; $j < $cFingerprints; $j++)
        {
            foreach ($rgTypes as $ixType)
            {
                $dblSum = 0;
                /* Calculate the Euclidean squared.
                 * This could have been in a method 
                 * call, but as we are inside our 
                 * third loop already, we'll keep the 
                 * overhead to a minimum.
                 */
                foreach ($rgoFingerprints[$ixType][$i][1] as $k => $dblValue1)
                {
                    $dblDiff = $dblValue1 - $rgoFingerprints[$ixType][$j][1][$k];
                    $dblSum += $dblDiff * $dblDiff;
                }
    
                // Is the Euclidean distance acceptably small?
                if ($dblSum < $dblMaxDistance[$ixType])
                {
                    // Only take the sqrt() if necessary
                    $dblEucl = sqrt($dblSum);
                    $ixFile1 = $rgoFingerprints[$ixType][$i][0];
                    $ixFile2 = $rgoFingerprints[$ixType][$j][0];
                    $dblScore = 1.0 - ($dblEucl / $rgMax[$ixType]);
                    if ($fCommaFirst)
                        $sOut .= ",\n";
                    else
                        $fCommaFirst = true;

                    $cInserts++;
                    $sOut .= "($ixFile1, $ixFile2, $ixType, $dblScore)";
                }
            }
        }
        // Let's have a maximum of 100 value groups in each INSERT.
        if ($cInserts >= 100)
        {
            fwrite($fp, $sOut);
            $sOut = '';
            fwrite($fp, ";\n\n");
            fwrite($fp, "INSERT DELAYED IGNORE INTO tblScore (ixFile1, ixFile2, ixType, dblScore) VALUES ");
            $fCommaFirst = false;
            echo "Wrote $cInserts inserts to file.\n";
            $cInserts = 0;
        }
    }
    
    // If we have something more to write, let's do it now.    
    fwrite($fp, $sOut);
    $sOut = '';
    fwrite($fp, ";\n\n");
    fclose($fp);
    echo "Done with everything.\n";
//}
?>
