<?php
/**
 * This file reads a list of files produced by Gunderson's descriptors,
 * and inserts all the fingerprints into tblFingerprint.
 * 
 * @author Vegard Andreas Larsen <vegarl@stud.ntnu.no>
 */
require_once('include/include.php');

// where do we find our files?
define('PATH_RESULTS', '/home/vegard/Masteroppgave/Kode/results/descriptors/');

$oDB = CDatabase::Get();

// Let's have a lookup list of the index of a fingerprint type.
// E.g. $rgixType['centroid'] == 6 
$sql = "SELECT sType, ixType FROM tblType";
$rgixType = $oDB->extended->GetAssoc($sql, null, null, null, MDB2_FETCHMODE_ASSOC, false);

// Read the list of files and remove excessive fat
$rgFiles = file(PATH_RESULTS.'filelist.txt');
$rgFiles = array_map('trim', $rgFiles);

// Our queries
$sqlInsert = "INSERT IGNORE INTO tblFingerprint (ixFile, ixType, sFingerprint) VALUES (?, ?, ?)";
$sqlFind = "SELECT ixFile FROM tblFile WHERE md5Path LIKE ?";

$i = 0;
echo "Starting...\n";
// Loop through files
foreach ($rgFiles as $md5Path)
{
    $rgDescriptors = file(PATH_RESULTS.$md5Path);
    $rgDescriptors = array_map('trim', $rgDescriptors);
    $ixFile = $oDB->extended->GetOne($sqlFind, null, array($md5Path), array('text'));
    foreach ($rgDescriptors as $line)
    {
        // Line structure as .ini files
        list($sDescriptor, $sValues) = explode('=', $line);

        /*
         * Some of the fingerprints have very large values, and this
         * is something Gunderson does in his Voronoi program. It should
         * have been done when the fingerprints were generated.
         * 
         * We are taking the Y-th root of any moments named *_momentY.
         */
        if (preg_match("/^(_moment)(\d)$/", $sDescriptor, $matches))
        {
            $rgValues = explode(',', $sValues);
            foreach ($rgValues as $k => $dbl)
            {
                if ($dbl < 0.0)
                    $rgValues[$k] = -pow(-$dbl, 1.0 / $matches[2]);
                else
                    $rgValues[$k] = pow($dbl, 1.0 / $matches[2]);
            }
            $sValues = implode(',', $rgValues);
        }
        // Do we know what kind of descriptor this is?
        // The fingerprint files also have a bunch of non-necesary data.
        if (isset($rgixType[$sDescriptor]))
        {
            $ixType = $rgixType[$sDescriptor];
            // Insert data into tblFingerprint
            $oDB->extended->execParam($sqlInsert, array($ixFile, $ixType, $sValues), array('integer', 'integer', 'text'));
        }
    }
    $i++;
    if ($i % 100 == 0)
        echo "\t[".date('H:i:s')."] Processed $i files\n";
}
echo "Done!\n";

?>
